@@ -1,3 +1,5 @@
+# SPDX-License-Identifier: MIT
+# Copyright (c) 2021-2025 Daniel Perna, SukramJ
 """
 Dynamic caches used at runtime by the central unit and clients.
 
@@ -25,13 +27,11 @@ from collections.abc import Mapping
 from datetime import datetime
 import logging
 from typing import Any, Final, cast
-from urllib.parse import unquote
 
 from aiohomematic import central as hmcu
 from aiohomematic.const import (
     DP_KEY_VALUE,
     INIT_DATETIME,
-    ISO_8859_1,
     LAST_COMMAND_SEND_STORE_TIMEOUT,
     MAX_CACHE_AGE,
     NO_CACHE_ENTRY,
@@ -310,10 +310,7 @@ class CentralDataCache:
 
     def add_data(self, interface: Interface, all_device_data: Mapping[str, Any]) -> None:
         """Add data to cache."""
-        self._value_cache[interface] = {
-            unquote(string=k, encoding=ISO_8859_1): unquote(string=v, encoding=ISO_8859_1) if isinstance(v, str) else v
-            for k, v in all_device_data.items()
-        }
+        self._value_cache[interface] = all_device_data
         self._refreshed_at[interface] = datetime.now()
 
     def get_data(
