@@ -1,3 +1,5 @@
+# SPDX-License-Identifier: MIT
+# Copyright (c) 2021-2025 Daniel Perna, SukramJ
 """
 Central unit and core orchestration for HomeMatic CCU and compatible backends.
 
@@ -119,6 +121,7 @@ from aiohomematic.const import (
     TIMEOUT,
     UN_IGNORE_WILDCARD,
     BackendSystemEvent,
+    CentralUnitState,
     DataOperationResult,
     DataPointCategory,
     DataPointKey,
@@ -146,7 +149,6 @@ from aiohomematic.exceptions import (
 from aiohomematic.model import create_data_points_and_events
 from aiohomematic.model.custom import CustomDataPoint, create_custom_data_points
 from aiohomematic.model.data_point import BaseParameterDataPoint, CallbackDataPoint
-from aiohomematic.model.decorators import info_property
 from aiohomematic.model.device import Channel, Device
 from aiohomematic.model.event import GenericEvent
 from aiohomematic.model.generic import GenericDataPoint
@@ -157,12 +159,21 @@ from aiohomematic.model.hub import (
     Hub,
     ProgramDpType,
 )
-from aiohomematic.model.support import PayloadMixin
-from aiohomematic.support import check_config, extract_exc_args, get_channel_no, get_device_address, get_ip_addr
+from aiohomematic.property_decorators import info_property
+from aiohomematic.support import (
+    LogContextMixin,
+    PayloadMixin,
+    check_config,
+    extract_exc_args,
+    get_channel_no,
+    get_device_address,
+    get_ip_addr,
+)
 
 __all__ = ["CentralConfig", "CentralUnit", "INTERFACE_EVENT_SCHEMA"]
 
 _LOGGER: Final = logging.getLogger(__name__)
+_LOGGER_EVENT: Final = logging.getLogger(f"{__package__}.event")
 
 # {central_name, central}
 CENTRAL_INSTANCES: Final[dict[str, CentralUnit]] = {}
@@ -179,12 +190,12 @@ INTERFACE_EVENT_SCHEMA = vol.Schema(
 )
 
 
-class CentralUnit(PayloadMixin):
+class CentralUnit(LogContextMixin, PayloadMixin):
     """Central unit that collects everything to handle communication from/to CCU/Homegear."""
 
     def __init__(self, central_config: CentralConfig) -> None:
         """Init the central unit."""
-        self._started: bool = False
+        self._state: CentralUnitState = CentralUnitState.NEW
         self._clients_started: bool = False
         self._device_add_semaphore: Final = asyncio.Semaphore()
         self._connection_state: Final = CentralConnectionState()
@@ -208,7 +219,7 @@ class CentralUnit(PayloadMixin):
         # {interface_id, client}
         self._clients: Final[dict[str, hmcl.Client]] = {}
         self._data_point_key_event_subscriptions: Final[
-            dict[DataPointKey, list[Callable[[Any], Coroutine[Any, Any, None]]]]
+            dict[DataPointKey, list[Callable[[Any, datetime], Coroutine[Any, Any, None]]]]
         ] = {}
         self._data_point_path_event_subscriptions: Final[dict[str, DataPointKey]] = {}
         self._sysvar_data_point_event_subscriptions: Final[dict[str, Callable]] = {}
@@ -233,7 +244,7 @@ class CentralUnit(PayloadMixin):
         self._hub: Hub = Hub(central=self)
         self._version: str | None = None
         # store last event received datetime by interface_id
-        self._last_events: Final[dict[str, datetime]] = {}
+        self._last_event_seen_for_interface: Final[dict[str, datetime]] = {}
         self._xml_rpc_callback_ip: str = IP_ANY_V4
         self._listen_ip_addr: str = IP_ANY_V4
         self._listen_port: int = PORT_ANY
@@ -248,7 +259,7 @@ class CentralUnit(PayloadMixin):
         """Return the xml rpc server callback ip address."""
         return self._xml_rpc_callback_ip
 
-    @info_property
+    @info_property(log_context=True)
     def url(self) -> str:
         """Return the central url."""
         return self._url
@@ -358,14 +369,14 @@ class CentralUnit(PayloadMixin):
         """Return the loop support."""
         return self._looper
 
-    @info_property
+    @info_property(log_context=True)
     def model(self) -> str | None:
         """Return the model of the backend."""
         if not self._model and (client := self.primary_client):
             self._model = client.model
         return self._model
 
-    @info_property
+    @info_property(log_context=True)
     def name(self) -> str:
         """Return the name of the backend."""
         return self._config.name
@@ -379,9 +390,9 @@ class CentralUnit(PayloadMixin):
         )
 
     @property
-    def started(self) -> bool:
-        """Return if the central is started."""
-        return self._started
+    def state(self) -> CentralUnitState:
+        """Return the central state."""
+        return self._state
 
     @property
     def supports_ping_pong(self) -> bool:
@@ -455,9 +466,17 @@ class CentralUnit(PayloadMixin):
     async def start(self) -> None:
         """Start processing of the central unit."""
 
-        if self._started:
+        _LOGGER.debug("START: Central %s is %s", self.name, self._state)
+        if self._state == CentralUnitState.INITIALIZING:
+            _LOGGER.debug("START: Central %s already starting", self.name)
+            return
+
+        if self._state == CentralUnitState.RUNNING:
             _LOGGER.debug("START: Central %s already started", self.name)
             return
+
+        self._state = CentralUnitState.INITIALIZING
+        _LOGGER.debug("START: Initializing Central %s", self.name)
         if self._config.enabled_interface_configs and (
             ip_addr := await self._identify_ip_addr(port=self._config.connection_check_port)
         ):
@@ -479,6 +498,7 @@ class CentralUnit(PayloadMixin):
                 self._listen_port = xml_rpc_server.listen_port
                 self._xml_rpc_server.add_central(self)
         except OSError as oserr:
+            self._state = CentralUnitState.STOPPED_BY_ERROR
             raise AioHomematicException(
                 f"START: Failed to start central unit {self.name}: {extract_exc_args(exc=oserr)}"
             ) from oserr
@@ -492,13 +512,24 @@ class CentralUnit(PayloadMixin):
             if self._config.enable_server:
                 self._start_scheduler()
 
-        self._started = True
+        self._state = CentralUnitState.RUNNING
+        _LOGGER.debug("START: Central %s is %s", self.name, self._state)
 
     async def stop(self) -> None:
         """Stop processing of the central unit."""
-        if not self._started:
+        _LOGGER.debug("STOP: Central %s is %s", self.name, self._state)
+        if self._state == CentralUnitState.STOPPING:
+            _LOGGER.debug("STOP: Central %s is already stopping", self.name)
+            return
+        if self._state == CentralUnitState.STOPPED:
+            _LOGGER.debug("STOP: Central %s is already stopped", self.name)
+            return
+        if self._state != CentralUnitState.RUNNING:
             _LOGGER.debug("STOP: Central %s not started", self.name)
             return
+        self._state = CentralUnitState.STOPPING
+        _LOGGER.debug("STOP: Stopping Central %s", self.name)
+
         await self.save_caches(save_device_descriptions=True, save_paramset_descriptions=True)
         self._stop_scheduler()
         await self._stop_clients()
@@ -522,8 +553,8 @@ class CentralUnit(PayloadMixin):
 
         # cancel outstanding tasks to speed up teardown
         self.looper.cancel_tasks()
-        # wait until tasks are finished
-        await self.looper.block_till_done()
+        # wait until tasks are finished (with wait_time safeguard)
+        await self.looper.block_till_done(wait_time=5.0)
 
         # Wait briefly for any auxiliary threads to finish without blocking forever
         max_wait_seconds = 5.0
@@ -532,7 +563,8 @@ class CentralUnit(PayloadMixin):
         while self._has_active_threads and waited < max_wait_seconds:
             await asyncio.sleep(interval)
             waited += interval
-        self._started = False
+        self._state = CentralUnitState.STOPPED
+        _LOGGER.debug("STOP: Central %s is %s", self.name, self._state)
 
     async def restart_clients(self) -> None:
         """Restart clients."""
@@ -1074,7 +1106,7 @@ class CentralUnit(PayloadMixin):
     @callback_event
     async def data_point_event(self, interface_id: str, channel_address: str, parameter: str, value: Any) -> None:
         """If a device emits some sort event, we will handle it here."""
-        _LOGGER.debug(
+        _LOGGER_EVENT.debug(
             "EVENT: interface_id = %s, channel_address = %s, parameter = %s, value = %s",
             interface_id,
             channel_address,
@@ -1084,7 +1116,7 @@ class CentralUnit(PayloadMixin):
         if not self.has_client(interface_id=interface_id):
             return
 
-        self.set_last_event_dt(interface_id=interface_id)
+        self.set_last_event_seen_for_interface(interface_id=interface_id)
         # No need to check the response of a XmlRPC-PING
         if parameter == Parameter.PONG:
             if "#" in value:
@@ -1108,11 +1140,12 @@ class CentralUnit(PayloadMixin):
 
         if dpk in self._data_point_key_event_subscriptions:
             try:
+                received_at = datetime.now()
                 for callback_handler in self._data_point_key_event_subscriptions[dpk]:
                     if callable(callback_handler):
-                        await callback_handler(value)
+                        await callback_handler(value, received_at)
             except RuntimeError as rterr:  # pragma: no cover
-                _LOGGER.debug(
+                _LOGGER_EVENT.debug(
                     "EVENT: RuntimeError [%s]. Failed to call callback for: %s, %s, %s",
                     extract_exc_args(exc=rterr),
                     interface_id,
@@ -1120,7 +1153,7 @@ class CentralUnit(PayloadMixin):
                     parameter,
                 )
             except Exception as exc:  # pragma: no cover
-                _LOGGER.warning(
+                _LOGGER_EVENT.warning(
                     "EVENT failed: Unable to call callback for: %s, %s, %s, %s",
                     interface_id,
                     channel_address,
@@ -1130,7 +1163,7 @@ class CentralUnit(PayloadMixin):
 
     def data_point_path_event(self, state_path: str, value: str) -> None:
         """If a device emits some sort event, we will handle it here."""
-        _LOGGER.debug(
+        _LOGGER_EVENT.debug(
             "DATA_POINT_PATH_EVENT: topic = %s, payload = %s",
             state_path,
             value,
@@ -1149,7 +1182,7 @@ class CentralUnit(PayloadMixin):
 
     def sysvar_data_point_path_event(self, state_path: str, value: str) -> None:
         """If a device emits some sort event, we will handle it here."""
-        _LOGGER.debug(
+        _LOGGER_EVENT.debug(
             "SYSVAR_DATA_POINT_PATH_EVENT: topic = %s, payload = %s",
             state_path,
             value,
@@ -1159,15 +1192,18 @@ class CentralUnit(PayloadMixin):
             try:
                 callback_handler = self._sysvar_data_point_event_subscriptions[state_path]
                 if callable(callback_handler):
-                    self._looper.create_task(callback_handler(value), name=f"sysvar-data-point-event-{state_path}")
+                    received_at = datetime.now()
+                    self._looper.create_task(
+                        callback_handler(value, received_at), name=f"sysvar-data-point-event-{state_path}"
+                    )
             except RuntimeError as rterr:  # pragma: no cover
-                _LOGGER.debug(
+                _LOGGER_EVENT.debug(
                     "EVENT: RuntimeError [%s]. Failed to call callback for: %s",
                     extract_exc_args(exc=rterr),
                     state_path,
                 )
             except Exception as exc:  # pragma: no cover
-                _LOGGER.warning(
+                _LOGGER_EVENT.warning(
                     "EVENT failed: Unable to call callback for: %s, %s",
                     state_path,
                     extract_exc_args(exc=exc),
@@ -1182,7 +1218,7 @@ class CentralUnit(PayloadMixin):
 
     def add_event_subscription(self, data_point: BaseParameterDataPoint) -> None:
         """Add data_point to central event subscription."""
-        if isinstance(data_point, (GenericDataPoint, GenericEvent)) and (
+        if isinstance(data_point, GenericDataPoint | GenericEvent) and (
             data_point.is_readable or data_point.supports_events
         ):
             if data_point.dpk not in self._data_point_key_event_subscriptions:
@@ -1194,13 +1230,13 @@ class CentralUnit(PayloadMixin):
             ):
                 self._data_point_path_event_subscriptions[data_point.state_path] = data_point.dpk
 
-    @inspector()
+    @inspector
     async def create_central_links(self) -> None:
         """Create a central links to support press events on all channels with click events."""
         for device in self.devices:
             await device.create_central_links()
 
-    @inspector()
+    @inspector
     async def remove_central_links(self) -> None:
         """Remove central links."""
         for device in self.devices:
@@ -1223,19 +1259,19 @@ class CentralUnit(PayloadMixin):
 
     def remove_event_subscription(self, data_point: BaseParameterDataPoint) -> None:
         """Remove event subscription from central collections."""
-        if isinstance(data_point, (GenericDataPoint, GenericEvent)) and data_point.supports_events:
+        if isinstance(data_point, GenericDataPoint | GenericEvent) and data_point.supports_events:
             if data_point.dpk in self._data_point_key_event_subscriptions:
                 del self._data_point_key_event_subscriptions[data_point.dpk]
             if data_point.state_path in self._data_point_path_event_subscriptions:
                 del self._data_point_path_event_subscriptions[data_point.state_path]
 
-    def get_last_event_dt(self, interface_id: str) -> datetime | None:
-        """Return the last event dt."""
-        return self._last_events.get(interface_id)
+    def get_last_event_seen_for_interface(self, interface_id: str) -> datetime | None:
+        """Return the last event seen for an interface."""
+        return self._last_event_seen_for_interface.get(interface_id)
 
-    def set_last_event_dt(self, interface_id: str) -> None:
-        """Set the last event dt."""
-        self._last_events[interface_id] = datetime.now()
+    def set_last_event_seen_for_interface(self, interface_id: str) -> None:
+        """Set the last event seen for an interface."""
+        self._last_event_seen_for_interface[interface_id] = datetime.now()
 
     async def execute_program(self, pid: str) -> bool:
         """Execute a program on CCU / Homegear."""
@@ -1621,7 +1657,7 @@ class _Scheduler(threading.Thread):
     async def _run_scheduler_tasks(self) -> None:
         """Run all tasks."""
         while self._active:
-            if not self._central.started:
+            if self._central.state != CentralUnitState.RUNNING:
                 _LOGGER.debug("SCHEDULER: Waiting till central %s is started", self._central.name)
                 await asyncio.sleep(SCHEDULER_NOT_STARTED_SLEEP)
                 continue
@@ -1678,7 +1714,7 @@ class _Scheduler(threading.Thread):
             _LOGGER.debug("REFRESH_CLIENT_DATA: Loading data for %s", self._central.name)
             for client in poll_clients:
                 await self._central.load_and_refresh_data_point_data(interface=client.interface)
-                self._central.set_last_event_dt(interface_id=client.interface_id)
+                self._central.set_last_event_seen_for_interface(interface_id=client.interface_id)
 
     @inspector(re_raise=False)
     async def _refresh_sysvar_data(self) -> None:
