@@ -1,3 +1,5 @@
+# SPDX-License-Identifier: MIT
+# Copyright (c) 2021-2025 Daniel Perna, SukramJ
 """
 Core data point model for AioHomematic.
 
@@ -63,24 +65,22 @@ from aiohomematic.context import IN_SERVICE_VAR
 from aiohomematic.decorators import get_service_calls
 from aiohomematic.exceptions import AioHomematicException, BaseHomematicException
 from aiohomematic.model import device as hmd
-from aiohomematic.model.decorators import cached_slot_property, config_property, state_property
 from aiohomematic.model.support import (
     DataPointNameData,
     DataPointPathData,
     GenericParameterType,
     PathData,
-    PayloadMixin,
     convert_value,
     generate_unique_id,
 )
-from aiohomematic.support import extract_exc_args
+from aiohomematic.property_decorators import cached_property, config_property, hm_property, state_property
+from aiohomematic.support import LogContextMixin, PayloadMixin, extract_exc_args, log_boundary_error
 
 __all__ = [
     "BaseDataPoint",
     "BaseParameterDataPoint",
     "CallParameterCollector",
     "CallbackDataPoint",
-    "EVENT_DATA_SCHEMA",
     "bind_collector",
 ]
 
@@ -131,7 +131,7 @@ EVENT_DATA_SCHEMA = vol.Schema(
 )
 
 
-class CallbackDataPoint(ABC):
+class CallbackDataPoint(ABC, LogContextMixin):
     """Base class for callback data point."""
 
     __slots__ = (
@@ -269,7 +269,7 @@ class CallbackDataPoint(ABC):
         """Return the data_point usage."""
         return DataPointUsage.DATA_POINT
 
-    @cached_slot_property
+    @cached_property
     def enabled_default(self) -> bool:
         """Return, if data_point should be enabled based on usage attribute."""
         return self.usage in (
@@ -295,12 +295,12 @@ class CallbackDataPoint(ABC):
         return self._path_data.state_path
 
     # @property
-    @cached_slot_property
+    @cached_property
     def service_methods(self) -> Mapping[str, Callable]:
         """Return all service methods."""
         return get_service_calls(obj=self)
 
-    @cached_slot_property
+    @cached_property
     def service_method_names(self) -> tuple[str, ...]:
         """Return all service methods."""
         return tuple(self.service_methods.keys())
@@ -447,7 +447,7 @@ class BaseDataPoint(CallbackDataPoint, PayloadMixin):
         """Return the availability of the device."""
         return self._device.available
 
-    @property
+    @hm_property(log_context=True)
     def channel(self) -> hmd.Channel:
         """Return the channel the data_point."""
         return self._channel
@@ -659,7 +659,7 @@ class BaseParameterDataPoint[
         """Return if the parameter is un ignored."""
         return self._is_un_ignored
 
-    @cached_slot_property
+    @cached_property
     def dpk(self) -> DataPointKey:
         """Return data_point key value."""
         return DataPointKey(
@@ -684,7 +684,7 @@ class BaseParameterDataPoint[
         """Return multiplier value."""
         return self._multiplier
 
-    @property
+    @hm_property(log_context=True)
     def parameter(self) -> str:
         """Return parameter name."""
         return self._parameter
@@ -699,7 +699,7 @@ class BaseParameterDataPoint[
         """Return raw unit value."""
         return self._raw_unit
 
-    @cached_slot_property
+    @cached_property
     def requires_polling(self) -> bool:
         """Return whether the data_point requires polling."""
         return not self._channel.device.client.supports_push_updates or (
@@ -785,7 +785,7 @@ class BaseParameterDataPoint[
         """Return the if data_point is visible in ccu."""
         return self._visible
 
-    @cached_slot_property
+    @cached_property
     def _enabled_by_channel_operation_mode(self) -> bool | None:
         """Return, if the data_point/event must be enabled."""
         if self._channel.type_name not in _CONFIGURABLE_CHANNEL:
@@ -856,7 +856,7 @@ class BaseParameterDataPoint[
         return f"{self._category}/{self._channel.device.model}/{self._parameter}"
 
     @abstractmethod
-    async def event(self, value: Any, received_at: datetime | None = None) -> None:
+    async def event(self, value: Any, received_at: datetime) -> None:
         """Handle event for which this handler has subscribed."""
 
     async def load_data_point_value(self, call_source: CallSource, direct_call: bool = False) -> None:
@@ -1085,12 +1085,24 @@ def bind_collector(
                     IN_SERVICE_VAR.reset(token)
                 in_service = IN_SERVICE_VAR.get()
                 if not in_service and log_level > logging.NOTSET:
-                    logging.getLogger(args[0].__module__).log(level=log_level, msg=extract_exc_args(exc=bhexc))
+                    context_obj = args[0]
+                    logger = logging.getLogger(context_obj.__module__)
+                    log_context = context_obj.log_context if isinstance(context_obj, LogContextMixin) else None
+                    # Reuse centralized boundary logging to ensure consistent 'extra' structure
+                    log_boundary_error(
+                        logger=logger,
+                        boundary="service",
+                        action=func.__name__,
+                        err=bhexc,
+                        level=log_level,
+                        log_context=log_context,
+                    )
+                # Re-raise domain-specific exceptions so callers and tests can handle them
+                raise
             else:
                 if token:
                     IN_SERVICE_VAR.reset(token)
                 return return_value
-            return None
 
         setattr(bind_wrapper, "ha_service", True)
         return bind_wrapper  # type: ignore[return-value]
