@@ -1,3 +1,5 @@
+# SPDX-License-Identifier: MIT
+# Copyright (c) 2021-2025 Daniel Perna, SukramJ
 """Support for data points used within aiohomematic."""
 
 from __future__ import annotations
@@ -5,6 +7,7 @@ from __future__ import annotations
 from abc import abstractmethod
 from collections.abc import Mapping
 from enum import StrEnum
+from functools import lru_cache
 import logging
 from typing import Any, Final
 
@@ -30,18 +33,12 @@ from aiohomematic.const import (
 )
 from aiohomematic.model import device as hmd
 from aiohomematic.model.custom import definition as hmed
-from aiohomematic.model.decorators import (
-    get_public_attributes_for_config_property,
-    get_public_attributes_for_info_property,
-    get_public_attributes_for_state_property,
-)
 from aiohomematic.support import to_bool
 
 __all__ = [
     "ChannelNameData",
     "DataPointNameData",
     "GenericParameterType",
-    "PayloadMixin",
     "check_channel_is_the_only_primary_channel",
     "convert_value",
     "generate_channel_unique_id",
@@ -67,39 +64,6 @@ _BINARY_SENSOR_TRUE_VALUE_DICT_FOR_VALUE_LIST: Final[Mapping[tuple[str, ...], st
 }
 
 
-class PayloadMixin:
-    """Mixin to add payload methods to class."""
-
-    __slots__ = ()
-
-    @property
-    def config_payload(self) -> Mapping[str, Any]:
-        """Return the config payload."""
-        return {
-            key: value
-            for key, value in get_public_attributes_for_config_property(data_object=self).items()
-            if value is not None
-        }
-
-    @property
-    def info_payload(self) -> Mapping[str, Any]:
-        """Return the info payload."""
-        return {
-            key: value
-            for key, value in get_public_attributes_for_info_property(data_object=self).items()
-            if value is not None
-        }
-
-    @property
-    def state_payload(self) -> Mapping[str, Any]:
-        """Return the state payload."""
-        return {
-            key: value
-            for key, value in get_public_attributes_for_state_property(data_object=self).items()
-            if value is not None
-        }
-
-
 class ChannelNameData:
     """Dataclass for channel name parts."""
 
@@ -533,7 +497,29 @@ def _check_channel_name_with_channel_no(name: str) -> bool:
 
 
 def convert_value(value: Any, target_type: ParameterType, value_list: tuple[str, ...] | None) -> Any:
-    """Convert a value to target_type."""
+    """
+    Convert a value to target_type with safe memoization.
+
+    To avoid redundant conversions across layers, we use an internal
+    LRU-cached helper for hashable inputs. For unhashable inputs, we
+    fall back to a direct conversion path.
+    """
+    # Normalize value_list to tuple to ensure hashability where possible
+    norm_value_list: tuple[str, ...] | None = tuple(value_list) if isinstance(value_list, list) else value_list
+    try:
+        # This will be cached if all arguments are hashable
+        return _convert_value_cached(value, target_type, norm_value_list)
+    except TypeError:
+        # Fallback non-cached path if any argument is unhashable
+        return _convert_value_noncached(value, target_type, norm_value_list)
+
+
+@lru_cache(maxsize=2048)
+def _convert_value_cached(value: Any, target_type: ParameterType, value_list: tuple[str, ...] | None) -> Any:
+    return _convert_value_noncached(value, target_type, value_list)
+
+
+def _convert_value_noncached(value: Any, target_type: ParameterType, value_list: tuple[str, ...] | None) -> Any:
     if value is None:
         return None
     if target_type == ParameterType.BOOL:
@@ -593,7 +579,7 @@ def get_index_of_value_from_value_list(
     value: SYSVAR_TYPE, value_list: tuple[str, ...] | list[str] | None
 ) -> int | None:
     """Check if value is in value list."""
-    if value is not None and isinstance(value, (str, StrEnum)) and value_list is not None and value in value_list:
+    if value is not None and isinstance(value, str | StrEnum) and value_list is not None and value in value_list:
         return value_list.index(value)
 
     return None
